<?php
// $Id: $

/**
 * @file
 * Include file with methods needed for display, including block content generation.
 */

/**
 * Load the modules of a given project.
 *
 * For each supported version of Drupal core, a best-match release is determined
 * and modules loaded for that release.
 *
 * @param $node
 *   The project node.
 * @return
 *   A nested array of modules keyed by core compatibility term ID.
 */
function project_info_project_modules_load($node) {
  // Allow a node ID argument.
  if (is_numeric($node)) {
    $node = node_load($node);
  }
  $modules = array();
  $versions = project_info_compatibility_list();
  foreach (array_keys($versions) as $tid) {
    if ($nid = project_info_get_primary_release($node->nid, $tid)) {
      if ($version_modules = project_info_release_modules_load($nid)) {
        $modules[$tid] = $version_modules;
      }
    }
  }

  return $modules;
}

/**
 * Load the modules of a given release.
 *
 * @param $nid
 *   The node ID of a release.
 * @return
 *   An array of modules.
 */
function project_info_release_modules_load($nid) {
  return(db_fetch_array(db_query("SELECT * FROM {project_info} WHERE nid = '%s'", $nid)));
}

/**
 * Load the content of project_info blocks.
 *
 * @param $delta
 *   The block to load.
 * @param $node
 *   The project node.
 * @return
 *   A block array.
 */
function project_info_block_view($delta, $node) {
  $block = array();
  switch ($delta) {
    case 'modules':
      $block['subject'] = t('Modules included in @title', array('@title' => $node->title));
      $block['content'] = project_info_modules($node);
    case 'dependencies':
      $block['subject'] = t('Projects that @title requires', array('@title' => $node->title));
      $block['content'] = project_info_dependencies($node);
    case 'dependents':
      $block['subject'] = t('Projects that require @title', array('@title' => $node->title));
      $block['content'] = project_info_dependents($node);
  }
  return $block;
}

/**
 * Return a list of modules for a given project.
 *
 * Modules are loaded for each relevent version of Drupal core.
 *
 * @param $node
 *   The project node.
 * @return
 *   A block array.
 */
function project_info_modules($node) {
  $output = '';
  $module_versions = project_info_modules_load($node->nid);
  $versions = project_info_compatibility_list();
  foreach ($versions as $version) {
    foreach ($module_versions as $tid => $modules) {
      $items = array();
      foreach ($modules as $module) {
        $items[] = theme('project_info_module', _project_info_module_preprocess($module));
      }
      if (count($items)) {
        $output .= theme('item_list', $items, check_plain($versions[$tid]));
      }
    }
  }
  return $output;
}

/**
 * Return a list of projects a given project depends on.
 *
 * @param $node
 *   The project node.
 * @return
 *   A block array.
 */
function project_info_dependencies($node) {
  $output = '';
  $versions = project_info_compatibility_list();
  foreach ($versions as $version) {
    foreach ($module_versions as $tid => $projects) {
      // TODO: finish writing this complex query.
      $result = db_query(db_rewrite_sql("SELECT DISTINCT(n.nid), n.title, n.created FROM {node} n INNER JOIN {project_release_nodes} prn ON n.nid = p.nid INNER JOIN {project_info} pi ON prn.nid = pi.nid INNER JOIN {project_info_dependency} pid ON pi.machine_name = pid.dependency INNER JOIN {project_info_dependency} pid2 ON ..... WHERE n.type = 'project_project' AND n.status = 1 ORDER BY n.created DESC"));
      if (db_num_rows($result)) {
        $output .= node_title_list($result, check_plain($versions[$tid]));
      }
    }
  }
  return $output;
}

/**
 * Return a list of projects dependent on a given project.
 *
 * @param $node
 *   The project node.
 * @return
 *   A block array.
 */
function project_info_dependents($node) {
  $output = '';
  $versions = project_info_compatibility_list();
  foreach ($versions as $version) {
    foreach ($module_versions as $tid => $projects) {
      // TODO: finish writing this complex query.
      $result = db_query(db_rewrite_sql("SELECT DISTINCT(n.nid), n.title, n.created FROM {node} n INNER JOIN {project_release_nodes} prn ON n.nid = p.nid INNER JOIN {project_info} pi ON prn.nid = pi.nid INNER JOIN {project_info_dependency} pid ON pi.machine_name = pid.dependency INNER JOIN {project_info_dependency} pid2 ON ..... WHERE n.type = 'project_project' AND n.status = 1 ORDER BY n.created DESC"));
      if (db_num_rows($result)) {
        $output .= node_title_list($result, check_plain($versions[$tid]));
      }
    }
  }
  return $output;
}

/**
 * Preprocess a module record in preparation for theming.
 *
 * Modules are loaded 
 *
 * @param $module
 *   A module array.
 * @return
 *   A processed module array.
 */
function _project_info_module_preprocess($module) {
  foreach (array('name', 'description') as $key) {
    $module[$key] = check_plain($module[$key]);
  }
  return $module;
}

/**
 * Return an array of core compatibility terms.
 *
 * @return
 *   An array keyed by term ID with term name values.
 */
function project_info_compatibility_list() {
  $versions = project_release_compatibility_list();
  // Unset the 'all' option.
  unset($versions[-1]);
  return $versions;
}

/**
 * Determine the primary release for a given project and a given version of core.
 *
 * Releases are ranked in order of recommended, supported, major, and minor release.
 * 
 * @param $pid
 *   The ID of the project.
 * @param $tid
 *   Drupal core compatibility term ID to match.
 */
function project_info_get_primary_release($pid, $tid) {
  return db_result(db_query_range("SELECT prn.nid FROM {project_release_nodes} prn INNER JOIN {project_release_supported_versions} prs ON prn.nid = prs.nid INNER JOIN {cvs_projects} c ON prn.pid = c.nid WHERE prn.pid = %d AND prs.tid = %d ORDER BY prs.recommended DESC, prs.supported DESC, prs.major DESC, prn.version_minor DESC", 0, 1, $pid, $tid));
}

/**
 * Theme a module record.
 *
 * @return
 *   Rendered HTML.
 */
function theme_project_info_module($module) {
  $output = '<strong>' . $module['name'] . '</strong>';
  $output .= '<br />';
  $output .= $module['description'];
  return $output;
}

