<?php
// $Id: $

/**
 * @file
 * Include file with methods needed for capturing project info data.
 * Intended to be called from a packaging script.
 */

/**
 * Parse Drupal info file format for Drupal 6+.
 *
 * Copied from drupal_parse_info_file() in Drupal core.
 *
 * @param $filename
 *   The file we are parsing. Accepts file with relative or absolute path.
 * @return
 *   The info array.
 * @todo
 *   Delete this function when upgrading to Drupal 6.
 */
function project_info_parse_info_file($filename) {
  $info = array();

  if (!file_exists($filename)) {
    return $info;
  }

  $data = file_get_contents($filename);
  if (preg_match_all('
    @^\s*                           # Start at the beginning of a line, ignoring leading whitespace
    ((?:
      [^=;\[\]]|                    # Key names cannot contain equal signs, semi-colons or square brackets,
      \[[^\[\]]*\]                  # unless they are balanced and not nested
    )+?)
    \s*=\s*                         # Key/value pairs are separated by equal signs (ignoring white-space)
    (?:
      ("(?:[^"]|(?<=\\\\)")*")|     # Double-quoted string, which may contain slash-escaped quotes/slashes
      (\'(?:[^\']|(?<=\\\\)\')*\')| # Single-quoted string, which may contain slash-escaped quotes/slashes
      ([^\r\n]*?)                   # Non-quoted string
    )\s*$                           # Stop at the next end of a line, ignoring trailing whitespace
    @msx', $data, $matches, PREG_SET_ORDER)) {
    foreach ($matches as $match) {
      // Fetch the key and value string
      $i = 0;
      foreach (array('key', 'value1', 'value2', 'value3') as $var) {
        $$var = isset($match[++$i]) ? $match[$i] : '';
      }
      $value = stripslashes(substr($value1, 1, -1)) . stripslashes(substr($value2, 1, -1)) . $value3;

      // Parse array syntax
      $keys = preg_split('/\]?\[/', rtrim($key, ']'));
      $last = array_pop($keys);
      $parent = &$info;

      // Create nested arrays
      foreach ($keys as $key) {
        if ($key == '') {
          $key = count($parent);
        }
        if (!isset($parent[$key]) || !is_array($parent[$key])) {
          $parent[$key] = array();
        }
        $parent = &$parent[$key];
      }

      // Handle PHP constants
      if (defined($value)) {
        $value = constant($value);
      }

      // Insert actual value
      if ($last == '') {
        $last = count($parent);
      }
      $parent[$last] = $value;
    }
  }

  return $info;
}

/**
 * Parse Drupal info file format for Drupal 6+.
 *
 * This function is designed to be called by a packaging script.
 *
 * @param $pid
 *   The ID of a project node.
 * @param $nid
 *   The ID of a project release node.
 * @param $tid
 *   The term ID of a project core compatibility term.
 * @param $file
 *   The .info file to be be processed.
 * @param $uri
 *   The short name of a project.
 * @param $version
 *   The version of the release being processed.
 */
function project_info_process_info_file($nid, $file, $uri, $version) {
  // Only process 6.x+ files.
  $matches = array();
  if (preg_match('/^((\d+)\.x)-.*/', $version, $matches) && $matches[2] >= 6) {
    // TODO: replace with drupal_parse_info_file() when upgrading to Drupal 6.
    $info = project_info_parse_info_file($file->filename);
    $info['machine_name'] = basename($file->filename, '.info');
    $info['nid'] = $nid;
    project_info_process_info($info);
  }
}

/**
 * Parse Drupal info file format for Drupal 6+.
 *
 * This function is designed to be called by a packaging script.
 * 
 * @param $file
 *   The .info file to be be processed.
 * @param $uri
 *   The short name of a project.
 * @param $version
 *   The version of the release being processed.
 */
function project_info_process_info($info) {
  // First save the item.
  project_info_save_item($info);
  // Save dependency information.
  if (isset($info['dependencies']) && is_array($info['dependencies'])) {
    $dependencies = array();
    foreach ($info['dependencies'] as $name) {
      $dependencies[] = $name,
    }
    project_info_save_item_dependencies($item['iid'], $dependencies);
  }
}

/**
 * Save a project system item (module, theme, etc.).
 * 
 * @param $info
 *   An info array as parsed from a .info file.
 * @return
 *   SAVED_UPDATED or SAVED_NEW.
 */
function project_info_save_item(&$item) {
  // First look for a match by ID.
  if (isset($item['iid'])) {
    $result = db_query("SELECT * FROM {project_info} WHERE iid = %d AND tid = %d", $item['iid'], $item['tid']);
  }
  // Then look for a match by name.
  elseif (isset($item['name'])) {
    $result = db_query("SELECT * FROM {project_info} WHERE name = '%s' AND tid = %d", $item['name'], $item['tid']);
  }
  if ($result) {
    $base_item = db_fetch_array($result);
  }
  else {
    $base_item = _project_info_item_defaults();
  }
  $item = array_merge($base_item, $item);
  // Update if we have an existing ID.
  if (isset($item['iid'])) {
    db_query("UPDATE {project_info} SET machine_name = '%s', pid = %d, name = '%s', type = '%s', tid = %d, description = '%s' WHERE iid = %d", $item['machine_name'], $item['name'], $item['type'], $item['tid'], $item['description'], $item['iid']);
    return SAVED_UPDATED;
  }
  // Otherwise, create a new record.
  else {
    $item['iid'] = db_next_id('{project_info}_iid');
    db_query("INSERT INTO {project_info} (iid, machine_name, pid, name, type, tid, description) VALUES (%d, '%s', %d, '%s', '%s', %d, '%s')", $item['iid'], $item['machine_name'], $item['name'], $item['type'], $item['tid'], $item['description']);
    return SAVED_NEW;
  }
}

/**
 * Return the default data for a project item.
 * 
 * @return
 *   An info array with default values.
 */
function _project_info_item_defaults() {
 return array(
   'machine_name' => '',
   'rid' => 0,
   'name' => '',
   'type' => '',
   'tid' => '',
   'description' => '',
 );
}

/**
 * Save dependency information for a module.
 * 
 * @param $iid
 *   The ID of the module.
 * @param $dependencies
 *   Array of modules that the module is dependent on.
 */
function project_info_save_item_dependencies($iid, $dependencies) {
  // Clear out old dependency data.
  db_query("DELETE FROM {project_info_dependency} WHERE iid = %d", $iid);
  if (is_array($dependencies)) {
    foreach ($dependencies as $dependency) {
      db_query("INSERT INTO {project_info_dependency} (iid, dependency) VALUES (%d, '%s')", $iid, $dependency);
    }
  }
}


